@@ -17,12 +17,17 @@ def _truncate_digest_for_dsa(dsa_cdata, digest, backend):
     """
     This function truncates digests that are longer than a given DS
     key's length so they can be signed. OpenSSL does this for us in
-    1.0.0c+ and it isn't needed in 0.9.8, but that leaves us with three
-    releases (1.0.0, 1.0.0a, and 1.0.0b) where this is a problem. This
-    truncation is not required in 0.9.8 because DSA is limited to SHA-1.
+    1.0.0c+, leaving us with three releases (1.0.0, 1.0.0a, and 1.0.0b) where
+    this is a problem.
     """
 
-    order_bits = backend._lib.BN_num_bits(dsa_cdata.q)
+    q = backend._ffi.new("BIGNUM **")
+    backend._lib.DSA_get0_pqg(
+        dsa_cdata, backend._ffi.NULL, q, backend._ffi.NULL
+    )
+    backend.openssl_assert(q[0] != backend._ffi.NULL)
+
+    order_bits = backend._lib.BN_num_bits(q[0])
     return _truncate_digest(digest, order_bits)
 
 
@@ -95,10 +100,17 @@ class _DSAParameters(object):
         self._dsa_cdata = dsa_cdata
 
     def parameter_numbers(self):
+        p = self._backend._ffi.new("BIGNUM **")
+        q = self._backend._ffi.new("BIGNUM **")
+        g = self._backend._ffi.new("BIGNUM **")
+        self._backend._lib.DSA_get0_pqg(self._dsa_cdata, p, q, g)
+        self._backend.openssl_assert(p[0] != self._backend._ffi.NULL)
+        self._backend.openssl_assert(q[0] != self._backend._ffi.NULL)
+        self._backend.openssl_assert(g[0] != self._backend._ffi.NULL)
         return dsa.DSAParameterNumbers(
-            p=self._backend._bn_to_int(self._dsa_cdata.p),
-            q=self._backend._bn_to_int(self._dsa_cdata.q),
-            g=self._backend._bn_to_int(self._dsa_cdata.g)
+            p=self._backend._bn_to_int(p[0]),
+            q=self._backend._bn_to_int(q[0]),
+            g=self._backend._bn_to_int(g[0])
         )
 
     def generate_private_key(self):
@@ -111,7 +123,13 @@ class _DSAPrivateKey(object):
         self._backend = backend
         self._dsa_cdata = dsa_cdata
         self._evp_pkey = evp_pkey
-        self._key_size = self._backend._lib.BN_num_bits(self._dsa_cdata.p)
+
+        p = self._backend._ffi.new("BIGNUM **")
+        self._backend._lib.DSA_get0_pqg(
+            dsa_cdata, p, self._backend._ffi.NULL, self._backend._ffi.NULL
+        )
+        self._backend.openssl_assert(p[0] != backend._ffi.NULL)
+        self._key_size = self._backend._lib.BN_num_bits(p[0])
 
     key_size = utils.read_only_property("_key_size")
 
@@ -119,40 +137,55 @@ class _DSAPrivateKey(object):
         return _DSASignatureContext(self._backend, self, signature_algorithm)
 
     def private_numbers(self):
+        p = self._backend._ffi.new("BIGNUM **")
+        q = self._backend._ffi.new("BIGNUM **")
+        g = self._backend._ffi.new("BIGNUM **")
+        pub_key = self._backend._ffi.new("BIGNUM **")
+        priv_key = self._backend._ffi.new("BIGNUM **")
+        self._backend._lib.DSA_get0_pqg(self._dsa_cdata, p, q, g)
+        self._backend.openssl_assert(p[0] != self._backend._ffi.NULL)
+        self._backend.openssl_assert(q[0] != self._backend._ffi.NULL)
+        self._backend.openssl_assert(g[0] != self._backend._ffi.NULL)
+        self._backend._lib.DSA_get0_key(self._dsa_cdata, pub_key, priv_key)
+        self._backend.openssl_assert(pub_key[0] != self._backend._ffi.NULL)
+        self._backend.openssl_assert(priv_key[0] != self._backend._ffi.NULL)
         return dsa.DSAPrivateNumbers(
             public_numbers=dsa.DSAPublicNumbers(
                 parameter_numbers=dsa.DSAParameterNumbers(
-                    p=self._backend._bn_to_int(self._dsa_cdata.p),
-                    q=self._backend._bn_to_int(self._dsa_cdata.q),
-                    g=self._backend._bn_to_int(self._dsa_cdata.g)
+                    p=self._backend._bn_to_int(p[0]),
+                    q=self._backend._bn_to_int(q[0]),
+                    g=self._backend._bn_to_int(g[0])
                 ),
-                y=self._backend._bn_to_int(self._dsa_cdata.pub_key)
+                y=self._backend._bn_to_int(pub_key[0])
             ),
-            x=self._backend._bn_to_int(self._dsa_cdata.priv_key)
+            x=self._backend._bn_to_int(priv_key[0])
         )
 
     def public_key(self):
-        dsa_cdata = self._backend._lib.DSA_new()
+        dsa_cdata = self._backend._lib.DSAparams_dup(self._dsa_cdata)
         self._backend.openssl_assert(dsa_cdata != self._backend._ffi.NULL)
         dsa_cdata = self._backend._ffi.gc(
             dsa_cdata, self._backend._lib.DSA_free
         )
-        dsa_cdata.p = self._backend._lib.BN_dup(self._dsa_cdata.p)
-        dsa_cdata.q = self._backend._lib.BN_dup(self._dsa_cdata.q)
-        dsa_cdata.g = self._backend._lib.BN_dup(self._dsa_cdata.g)
-        dsa_cdata.pub_key = self._backend._lib.BN_dup(self._dsa_cdata.pub_key)
+        pub_key = self._backend._ffi.new("BIGNUM **")
+        self._backend._lib.DSA_get0_key(
+            self._dsa_cdata, pub_key, self._backend._ffi.NULL
+        )
+        self._backend.openssl_assert(pub_key[0] != self._backend._ffi.NULL)
+        pub_key_dup = self._backend._lib.BN_dup(pub_key[0])
+        res = self._backend._lib.DSA_set0_key(
+            dsa_cdata, pub_key_dup, self._backend._ffi.NULL
+        )
+        self._backend.openssl_assert(res == 1)
         evp_pkey = self._backend._dsa_cdata_to_evp_pkey(dsa_cdata)
         return _DSAPublicKey(self._backend, dsa_cdata, evp_pkey)
 
     def parameters(self):
-        dsa_cdata = self._backend._lib.DSA_new()
+        dsa_cdata = self._backend._lib.DSAparams_dup(self._dsa_cdata)
         self._backend.openssl_assert(dsa_cdata != self._backend._ffi.NULL)
         dsa_cdata = self._backend._ffi.gc(
             dsa_cdata, self._backend._lib.DSA_free
         )
-        dsa_cdata.p = self._backend._lib.BN_dup(self._dsa_cdata.p)
-        dsa_cdata.q = self._backend._lib.BN_dup(self._dsa_cdata.q)
-        dsa_cdata.g = self._backend._lib.BN_dup(self._dsa_cdata.g)
         return _DSAParameters(self._backend, dsa_cdata)
 
     def private_bytes(self, encoding, format, encryption_algorithm):
@@ -164,6 +197,11 @@ class _DSAPrivateKey(object):
             self._dsa_cdata
         )
 
+    def sign(self, data, algorithm):
+        signer = self.signer(algorithm)
+        signer.update(data)
+        return signer.finalize()
+
 
 @utils.register_interface(dsa.DSAPublicKeyWithSerialization)
 class _DSAPublicKey(object):
@@ -171,34 +209,50 @@ class _DSAPublicKey(object):
         self._backend = backend
         self._dsa_cdata = dsa_cdata
         self._evp_pkey = evp_pkey
-        self._key_size = self._backend._lib.BN_num_bits(self._dsa_cdata.p)
+        p = self._backend._ffi.new("BIGNUM **")
+        self._backend._lib.DSA_get0_pqg(
+            dsa_cdata, p, self._backend._ffi.NULL, self._backend._ffi.NULL
+        )
+        self._backend.openssl_assert(p[0] != backend._ffi.NULL)
+        self._key_size = self._backend._lib.BN_num_bits(p[0])
 
     key_size = utils.read_only_property("_key_size")
 
     def verifier(self, signature, signature_algorithm):
+        if not isinstance(signature, bytes):
+            raise TypeError("signature must be bytes.")
+
         return _DSAVerificationContext(
             self._backend, self, signature, signature_algorithm
         )
 
     def public_numbers(self):
+        p = self._backend._ffi.new("BIGNUM **")
+        q = self._backend._ffi.new("BIGNUM **")
+        g = self._backend._ffi.new("BIGNUM **")
+        pub_key = self._backend._ffi.new("BIGNUM **")
+        self._backend._lib.DSA_get0_pqg(self._dsa_cdata, p, q, g)
+        self._backend.openssl_assert(p[0] != self._backend._ffi.NULL)
+        self._backend.openssl_assert(q[0] != self._backend._ffi.NULL)
+        self._backend.openssl_assert(g[0] != self._backend._ffi.NULL)
+        self._backend._lib.DSA_get0_key(
+            self._dsa_cdata, pub_key, self._backend._ffi.NULL
+        )
+        self._backend.openssl_assert(pub_key[0] != self._backend._ffi.NULL)
         return dsa.DSAPublicNumbers(
             parameter_numbers=dsa.DSAParameterNumbers(
-                p=self._backend._bn_to_int(self._dsa_cdata.p),
-                q=self._backend._bn_to_int(self._dsa_cdata.q),
-                g=self._backend._bn_to_int(self._dsa_cdata.g)
+                p=self._backend._bn_to_int(p[0]),
+                q=self._backend._bn_to_int(q[0]),
+                g=self._backend._bn_to_int(g[0])
             ),
-            y=self._backend._bn_to_int(self._dsa_cdata.pub_key)
+            y=self._backend._bn_to_int(pub_key[0])
         )
 
     def parameters(self):
-        dsa_cdata = self._backend._lib.DSA_new()
-        self._backend.openssl_assert(dsa_cdata != self._backend._ffi.NULL)
+        dsa_cdata = self._backend._lib.DSAparams_dup(self._dsa_cdata)
         dsa_cdata = self._backend._ffi.gc(
             dsa_cdata, self._backend._lib.DSA_free
         )
-        dsa_cdata.p = self._backend._lib.BN_dup(self._dsa_cdata.p)
-        dsa_cdata.q = self._backend._lib.BN_dup(self._dsa_cdata.q)
-        dsa_cdata.g = self._backend._lib.BN_dup(self._dsa_cdata.g)
         return _DSAParameters(self._backend, dsa_cdata)
 
     def public_bytes(self, encoding, format):
@@ -210,6 +264,12 @@ class _DSAPublicKey(object):
         return self._backend._public_key_bytes(
             encoding,
             format,
+            self,
             self._evp_pkey,
             None
         )
+
+    def verify(self, signature, data, algorithm):
+        verifier = self.verifier(signature, algorithm)
+        verifier.update(data)
+        verifier.verify()
